﻿using Confluent.Kafka;
using Microsoft.Extensions.Configuration;
using Newtonsoft.Json.Linq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace EventListener
{

    public class KafkaConsumer
    {
        private KafkaHttpClient _client;
        private IConfigurationRoot _configuration;
        public KafkaConsumer(KafkaHttpClient client, IConfigurationRoot configuration)
        { 
            _client = client;
            _configuration = configuration;
        }
        public void ConsumeMessages(string topic)
        {       

            var config = new ConsumerConfig
            {
                BootstrapServers = _configuration["BootstrapServers"],  //"onlineauctionkafkahub.servicebus.windows.net:9093",
                SecurityProtocol = SecurityProtocol.SaslSsl,
                SocketTimeoutMs = 60000,                //this corresponds to the Consumer config `request.timeout.ms`
                SessionTimeoutMs = 30000,
                SaslMechanism = SaslMechanism.Plain,
                SaslUsername = "$ConnectionString",
                SaslPassword = _configuration["SaslPassword"],  // "Endpoint=sb://onlineauctionkafkahub.servicebus.windows.net/;SharedAccessKeyName=RootManageSharedAccessKey;SharedAccessKey=m0JYP94laWQ2LUtXA7j8CJFPUKOPGe5yd9LlYl2lsOo=",
                //SslCaLocation = @".\cacert.pem",
                GroupId = _configuration["GroupId"],  // "onlineauctiongroup",
                EnableSslCertificateVerification=false,
                AutoOffsetReset = AutoOffsetReset.Earliest,
                BrokerVersionFallback = "1.0.0",        //Event Hubs for Kafka Ecosystems supports Kafka v1.0+, a fallback to an older API will fail
            };
            using (var consumer = new ConsumerBuilder<Ignore, string>(config).Build())
            {
               
                Logger.Instance.LogMessage("Subscribing topic");
                consumer.Subscribe(topic);
                
                CancellationTokenSource cts = new CancellationTokenSource();
                Console.CancelKeyPress += (_, e) => {
                    e.Cancel = true; // prevent the process from terminating.
                    cts.Cancel();
                };
                try
                {
                    while (true)
                    {
                       try
                        {
                            var cr = consumer.Consume(cts.Token);
                            Logger.Instance.LogMessage($"Got one message value is {cr.Value}");
                            dynamic data = JObject.Parse(cr.Value);
                            string bidId = data.bidid;
                            string customerId = data.userId;
                            decimal bidAmount = Convert.ToDecimal(data.bidAmount);
                            int auctionId = Convert.ToInt32(data.auctionId);

                            Logger.Instance.LogMessage($"Calling Update Auction for Bid for {cr.Value}");
                            _client.UpdateAuctionForBid(cr.Value);

                            Logger.Instance.LogMessage($"Consumed message '{cr.Value}' at: '{cr.TopicPartitionOffset}'.");

                            //Console.WriteLine($"Consumed message '{cr.Value}' at: '{cr.TopicPartitionOffset}'.");
                        }
                        catch (ConsumeException e)
                        {

                            Logger.Instance.LogMessage($"Error occured: {e.Message} Stack Trace: {e.StackTrace}");
                            //Console.WriteLine($"Error occured: {e.Error.Reason}");
                        }
                    }
                }
                catch (OperationCanceledException)
                {
                    // Ensure the consumer leaves the group cleanly and final offsets are committed.
                    consumer.Close();
                }
            }

        }

    }
}
